var ts = require("typescript");
var path = require("path");
var jsdoc = require("./jsdoc");
var processor = require("./processor");
function isDeclarationNode(node) {
    var blocksKind = [
        183 /* VariableDeclaration */,
        184 /* FunctionDeclaration */,
        185 /* ClassDeclaration */,
        186 /* InterfaceDeclaration */,
        187 /* TypeAliasDeclaration */,
        188 /* EnumDeclaration */,
        189 /* ModuleDeclaration */,
        191 /* ImportDeclaration */
    ];
    return blocksKind.indexOf(node.kind) !== -1;
}
function createComments(node, checker, result) {
    if (result === void 0) { result = new jsdoc.CommentsList(); }
    processor.process(result, node, checker);
    ts.forEachChild(node, function (child) {
        createComments(child, checker, result);
    });
    return result;
}
function getOptionsFromConfig(opts) {
    if (!opts) {
        return;
    }
    var target = opts.target;
    if (target) {
        if (target === "latest") {
            target = "Latest";
        }
        else if (target !== "Latest") {
            target = target.toUpperCase();
        }
        opts.target = ts.ScriptTarget[target];
    }
    var mod = opts.module;
    switch (mod) {
        case "commonjs":
        case "CommonJS":
        case "Commonjs":
        case "CommonJs":
            opts.module = 1 /* CommonJS */;
            break;
        case "amd":
        case "AMD":
        case "Amd":
            opts.module = 2 /* AMD */;
            break;
        default:
            opts.module = 0 /* None */;
            break;
    }
    return opts;
}
exports.getOptionsFromConfig = getOptionsFromConfig;
function getFileComments(filename, options) {
    filename = filename.replace(/\\/g, "/");
    options = options || {
        target: 2 /* ES6 */,
        module: 1 /* CommonJS */
    };
    var host = ts.createCompilerHost(options), program = ts.createProgram([filename], options, host), checker = program.getTypeChecker(true), file = program.getSourceFile(filename);
    return createComments(file, checker);
}
exports.getFileComments = getFileComments;
function compile(program, commandLine, checker) {
    commandLine.filenames.forEach(function (filename) {
        var options = commandLine.options, sourceFile = program.getSourceFile(filename), comments = createComments(sourceFile, checker), newFileName = sourceFile.filename.replace(/(\.d)?\.ts$/, ".jsdoc");
        if (options.outDir) {
            newFileName = newFileName.replace(path.dirname(sourceFile.filename), options.outDir);
        }
        else if (options.out) {
            newFileName = options.out;
        }
        ts.sys.writeFile(newFileName, comments.toString());
    });
}
exports.compile = compile;
