var ts = require("typescript");
var syntax = require("./syntax");
function getKindsToProcess() {
    var self = getKindsToProcess;
    if (!self.result) {
        self.result = [
            124 /* Property */,
            125 /* Method */,
            126 /* Constructor */,
            127 /* GetAccessor */,
            128 /* SetAccessor */,
            183 /* VariableDeclaration */,
            184 /* FunctionDeclaration */,
            185 /* ClassDeclaration */,
            186 /* InterfaceDeclaration */,
            187 /* TypeAliasDeclaration */,
            188 /* EnumDeclaration */,
            189 /* ModuleDeclaration */,
            191 /* ImportDeclaration */,
            192 /* ExportAssignment */
        ];
    }
    return self.result;
}
function hasToProcess(node) {
    var kinds = getKindsToProcess();
    return kinds.indexOf(node.kind) !== -1;
}
function process(comments, node, checker) {
    if (!hasToProcess(node)) {
        return;
    }
    switch (node.kind) {
        case 183 /* VariableDeclaration */:
        case 124 /* Property */:
            syntax.forVar(comments, node, checker);
            break;
        case 125 /* Method */:
        case 184 /* FunctionDeclaration */:
            syntax.forFunction(comments, node, checker);
            break;
        case 127 /* GetAccessor */:
        case 128 /* SetAccessor */:
            syntax.forAccessor(comments, node, checker);
            break;
        case 186 /* InterfaceDeclaration */:
            syntax.forInterface(comments, node, checker);
            break;
        case 185 /* ClassDeclaration */:
            syntax.forClass(comments, node, checker);
            break;
        case 126 /* Constructor */:
            syntax.forConstructor(comments, node, checker);
            break;
        case 187 /* TypeAliasDeclaration */:
            syntax.forType(comments, node, checker);
            break;
        case 188 /* EnumDeclaration */:
            syntax.forEnum(comments, node, checker);
            break;
        case 189 /* ModuleDeclaration */:
            syntax.forModule(comments, node, checker);
            break;
        case 192 /* ExportAssignment */:
            syntax.forExport(comments, node, checker);
            break;
    }
    //return comments;
}
exports.process = process;
