var ts = require("typescript");
var common = require("./common");
var utils = require("./utils");
function forVar(list, node, checker) {
    var parent = node.parent, comments;
    if (parent.kind === 136 /* TypeLiteral */) {
        var anonymousName = utils.getAnonymousName(parent);
        comments = list.get("typedef", anonymousName, utils.getParentName(parent.parent, checker));
        if (!comments) {
            comments = list.get("interface", anonymousName, utils.getParentModuleName(parent, checker));
        }
        common.addTypeDefProperty(comments, "property", node, checker);
    }
    else {
        comments = list.addFromNode(node);
        common.addName(comments, "member", node, checker);
        common.addDefault(comments, node);
        common.addParent(comments, node, checker);
        common.addModifier(comments, node);
    }
}
exports.forVar = forVar;
function forFunction(list, node, checker) {
    var comments = list.addFromNode(node), signature = checker.getSignatureFromDeclaration(node);
    common.addName(comments, "function", node);
    common.addVariation(comments, node, checker);
    common.addGeneric(comments, node);
    common.addFromSignature(comments, node, signature, checker);
    common.addParent(comments, node, checker);
    common.addModifier(comments, node);
}
exports.forFunction = forFunction;
function forAccessor(list, node, checker) {
    //console.log(node.getText());
}
exports.forAccessor = forAccessor;
function forInterface(list, node, checker) {
    var comments = list.addFromNode(node), type = checker.getTypeAtLocation(node), callSignatures = type.getCallSignatures(), ctorSignatures = type.getConstructSignatures();
    if (node.members.length === callSignatures.length) {
        common.addCallbacks(comments, node, callSignatures, checker);
        comments.clean();
    }
    else {
        common.addName(comments, "interface", node);
        common.addGeneric(comments, node);
        common.addImplementations(comments, node, checker);
        common.addParent(comments, node, checker);
        common.addCallSignatures(comments, node, "ctor", ctorSignatures, checker);
        common.addCallSignatures(comments, node, "call", callSignatures, checker);
        common.merge(comments, node, checker);
    }
}
exports.forInterface = forInterface;
function forClass(list, node, checker) {
    var comments = list.addFromNode(node);
    common.addName(comments, "class", node);
    common.addVariation(comments, node, checker);
    common.addGeneric(comments, node);
    common.addImplementations(comments, node, checker);
    common.addParent(comments, node, checker);
}
exports.forClass = forClass;
function forConstructor(list, node, checker) {
    var comments = list.addFromNode(node), signature = checker.getSignatureFromDeclaration(node);
    comments.getOrAddTag("ctor");
    common.addVariation(comments, node, checker);
    common.addParent(comments, node, checker);
    common.addFromSignature(comments, node, signature, checker);
}
exports.forConstructor = forConstructor;
function forType(list, node, checker) {
    var comments = list.addFromNode(node);
    common.addName(comments, "typedef", node, checker);
    common.addVariation(comments, node, checker);
    common.addParent(comments, node, checker);
}
exports.forType = forType;
function forEnum(list, node, checker) {
    var comments = list.addFromNode(node);
    common.addName(comments, "enum", node);
    common.addModifier(comments, node);
    common.addParent(comments, node, checker);
    common.addEnumMembers(comments, node, checker);
}
exports.forEnum = forEnum;
function forModule(list, node, checker) {
    var comments = list.addFromNode(node);
    if (utils.isExternal(node)) {
        //common.addName(comments, "namespace", node);
        comments.getOrAddTag("module").name = node.name.text.replace(/"/g, "");
        comments.getOrAddTag("global");
        common.merge(comments, node, checker);
    }
    else {
        common.addName(comments, "namespace", node);
        common.addParent(comments, node, checker);
        common.merge(comments, node, checker);
    }
}
exports.forModule = forModule;
function forExport(list, node, checker) {
    //console.log(node.getText());
}
exports.forExport = forExport;
