var ts = require("typescript");
var utils = require("./utils");
function getIntrinsicResult(type, checker) {
    return {
        name: checker.typeToString(type),
        anonymous: []
    };
}
function getAnonymousResult(node, typeNode, checker) {
    return {
        name: utils.formatType(utils.getAnonymousFullName(node, checker)),
        anonymous: [typeNode]
    };
}
function getLinkedResult(typeNode) {
    return {
        name: utils.formatType(typeNode.getText()),
        anonymous: []
    };
}
function getUnionTypeResult(node, typeNode, checker) {
    var result = { name: "", anonymous: [] };
    result.name = typeNode.types.map(function (typeArg) {
        var tmp = getNodeTypeName(node, typeArg, checker);
        result.anonymous = result.anonymous.concat(tmp.anonymous);
        return tmp.name;
    }).join(" | ");
    return result;
}
function getArrayResult(node, typeNode, checker) {
    var result = getNodeTypeName(node, typeNode.elementType, checker);
    result.name = utils.formatType(result.name + "[]");
    return result;
}
function getGenericPart(node, typeNode, checker) {
    var result, typeTemp, typeArgs, typeParams;
    if (typeArgs = typeNode.typeArguments) {
        result = { name: "", anonymous: [] };
        result.name = typeArgs.map(function (typeArg) {
            var tmp = getNodeTypeName(node, typeArg, checker);
            result.anonymous = result.anonymous.concat(tmp.anonymous);
            return tmp.name;
        }).join(", ");
        result.name = "<" + result.name + ">";
        return result;
    }
    if (typeParams = node.typeParameters) {
        return {
            name: "<" + typeParams.map(utils.getNodeName).join(", ") + ">",
            anonymous: []
        };
    }
}
function getNodeTypeName(node, typeNode, checker, givenType) {
    if (typeNode.getText().indexOf("typeof") === 0) {
        return getLinkedResult(typeNode);
    }
    var type = typeNode ? checker.getTypeAtLocation(typeNode) : givenType, symbol = type.symbol, tmp, result;
    if (utils.hasFlag(127 /* Intrinsic */, type.flags) || utils.hasFlag(type.flags, 512 /* TypeParameter */)) {
        return getIntrinsicResult(type, checker);
    }
    if (!typeNode) {
        typeNode = symbol.valueDeclaration.type;
    }
    if (utils.hasFlag(type.flags, 32768 /* Anonymous */)) {
        return getAnonymousResult(node, typeNode, checker);
    }
    if (utils.hasFlag(type.flags, 16384 /* Union */) && typeNode.types) {
        return getUnionTypeResult(node, typeNode, checker);
    }
    if (typeNode.kind === 137 /* ArrayType */) {
        return getArrayResult(node, typeNode, checker);
    }
    result = {
        name: symbol ? checker.getFullyQualifiedName(symbol).replace(/"/g, "") : checker.typeToString(type),
        anonymous: []
    };
    tmp = getGenericPart(node, typeNode, checker);
    if (tmp) {
        result.name += tmp.name;
        result.anonymous = tmp.anonymous;
    }
    result.name = utils.formatType(result.name);
    return result;
}
exports.getNodeTypeName = getNodeTypeName;
