var ts = require("typescript");
var isExternalRegexp = /"[^"]+"/, fixRepeatable = /^\.\.\.(?:Array\.<)?(.+)(?:(?:>)|(?:\[\]))/g, fixGenerics = /([^<\.]+)<([^>]+)>/g, fixGenericsArrays = /([^<]+)\.<(.+)>\[\]$/g;
function getNodeName(node) {
    var decla = node;
    if (!decla || !("name" in decla) || ("expression" in decla.name)) {
        return null;
    }
    return decla.name.text;
}
exports.getNodeName = getNodeName;
function getDefault(node) {
    var initializer = node.initializer;
    if (!initializer) {
        return null;
    }
    return initializer.text || initializer.getText();
}
exports.getDefault = getDefault;
function getParentModuleName(node, checker) {
    while (node.parent.kind !== 189 /* ModuleDeclaration */ && node.parent.kind !== 201 /* SourceFile */) {
        node = node.parent;
    }
    if (node.parent.kind === 189 /* ModuleDeclaration */) {
        var symbol = checker.getTypeAtLocation(node.parent).getSymbol(), typeName = symbol ? checker.getFullyQualifiedName(symbol).replace(/"/g, "") : node.parent.name.text.replace(/"/g, "");
        return (isExternal(node.parent) ? "module:" : "") + typeName;
    }
    else {
        var mod = node.amdModuleName;
        if (mod) {
            return "module:" + mod;
        }
    }
    return null;
}
exports.getParentModuleName = getParentModuleName;
function getParentName(node, checker) {
    var symbol;
    var kinds = [
        185 /* ClassDeclaration */,
        186 /* InterfaceDeclaration */,
        189 /* ModuleDeclaration */,
        188 /* EnumDeclaration */,
        201 /* SourceFile */,
        136 /* TypeLiteral */
    ];
    while (kinds.indexOf(node.parent.kind) === -1) {
        node = node.parent;
    }
    if (node.parent.kind === 185 /* ClassDeclaration */ || node.parent.kind === 186 /* InterfaceDeclaration */ || node.parent.kind === 188 /* EnumDeclaration */) {
        symbol = checker.getTypeAtLocation(node.parent).getSymbol();
        return checker.getFullyQualifiedName(symbol).replace(/"/g, "");
    }
    else if (node.parent.kind === 189 /* ModuleDeclaration */) {
        symbol = checker.getTypeAtLocation(node.parent).getSymbol();
        var typeName = symbol ? checker.getFullyQualifiedName(symbol).replace(/"/g, "") : node.parent.name.text.replace(/"/g, "");
        return (isExternal(node.parent) ? "module:" : "") + typeName;
    }
    else if (node.parent.kind === 136 /* TypeLiteral */) {
        return getAnonymousFullName(node.parent, checker);
    }
    else {
        return node.amdModuleName;
    }
    return null;
}
exports.getParentName = getParentName;
function getAnonymousName(node) {
    var oldNode = node, nodeType = node.type, nodeName = getNodeName(node), tagName = "", parent;
    var kinds = [
        183 /* VariableDeclaration */,
        124 /* Property */,
        129 /* CallSignature */,
        130 /* ConstructSignature */,
        184 /* FunctionDeclaration */,
        125 /* Method */,
        123 /* Parameter */
    ];
    while (kinds.indexOf(node.kind) === -1) {
        node = node.parent;
        nodeName = getNodeName(node);
        if (!node) {
            var msg = "Unknow scenario during anonymous name construction.\n" + "Node kind:\t" + getSyntaxKindString(oldNode) + "\n" + "Node text:\t" + oldNode.getText();
            var err = new Error(msg);
            err.node = node;
            throw err;
        }
    }
    if (node.kind === 184 /* FunctionDeclaration */ || node.kind === 125 /* Method */ || node.kind === 130 /* ConstructSignature */ || node.kind === 129 /* CallSignature */) {
        tagName = "Returns";
    }
    else if (node.kind === 123 /* Parameter */) {
        tagName = nodeName.substr(0, 1).toUpperCase() + nodeName.substr(1);
        nodeName = getNodeName(node.parent);
        if (!nodeName) {
            if (node.parent.kind === 130 /* ConstructSignature */ || node.parent.kind === 126 /* Constructor */) {
                nodeName = "Construct";
            }
            else if (node.parent.kind === 129 /* CallSignature */) {
                nodeName = "Call";
            }
            else {
                nodeName = getAnonymousName(node.parent.parent);
            }
        }
    }
    //else if (node.kind === ts.SyntaxKind.VariableDeclaration || node.kind === ts.SyntaxKind.Property) {
    //    //tagName = isCallback ? "Callback" : "Type";
    //}
    nodeName = nodeName.substr(0, 1).toUpperCase() + nodeName.substr(1);
    if (nodeType && nodeType.parameters) {
        tagName += "Callback";
    }
    else {
        tagName += "Type";
    }
    return nodeName + tagName;
}
exports.getAnonymousName = getAnonymousName;
function getAnonymousFullName(node, checker) {
    var parentName = getParentName(node, checker), anonymousName = getAnonymousName(node);
    if (parentName && parentName !== "null") {
        return parentName + "." + anonymousName;
    }
    else {
        return anonymousName;
    }
}
exports.getAnonymousFullName = getAnonymousFullName;
function formatType(type) {
    if (!type) {
        type = "any";
    }
    type = type.replace(fixRepeatable, "...$1");
    type = type.replace(fixGenerics, "$1.<$2>");
    type = type.replace(fixGenericsArrays, "Array.<$1.<$2>>");
    type = type.replace("typeof", "@link");
    return type;
}
exports.formatType = formatType;
function formatParam(name, node) {
    if (!node) {
        return name;
    }
    if (node.dotDotDotToken) {
        name = "..." + name;
    }
    if (node.questionToken || node.initializer) {
        name = "[" + name;
        if (node.initializer) {
            name += "=" + node.initializer.text || node.initializer.getText();
        }
        name += "]";
    }
    return name;
}
exports.formatParam = formatParam;
function isExternal(mod) {
    return isExternalRegexp.test(mod.name.getText());
}
exports.isExternal = isExternal;
function getComments(symbol) {
    var comments = symbol.getDocumentationComment();
    if (comments && comments.length > 0) {
        return comments.map(function (part) { return part.text; }).join("").replace(/(\r?\n)/g, "$1 * ");
    }
}
exports.getComments = getComments;
function getSyntaxKindString(node) {
    return ts.SyntaxKind[node.kind];
}
exports.getSyntaxKindString = getSyntaxKindString;
function hasFlag(flags, flag) {
    return ((flags & flag) === flag);
}
exports.hasFlag = hasFlag;
